"""
URL configuration
"""

from django.urls import include, path
from rest_framework import routers

import ichec_django_core.urls
import marinerg_facility.urls
import marinerg_test_access.urls
import marinerg_data_access.urls

router = routers.DefaultRouter()
ichec_django_core.urls.register_drf_views(router)
marinerg_facility.urls.register_drf_views(router)
marinerg_test_access.urls.register_drf_views(router)
marinerg_data_access.urls.register_drf_views(router)

urlpatterns = [
    path("api/", include(router.urls)),
    path("api/", include("marinerg_test_access.urls")),
    path("api/", include("marinerg_facility.urls")),
    path("", include("ichec_django_core.urls")),
]



from functools import partial

import PIL

from django.contrib.auth.models import Group, Permission
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User
from rest_framework import status
from rest_framework.test import APITestCase

from api.models import PortalMember, AccessCall, AccessApplication


def generate_image():
    return PIL.Image.linear_gradient("L")


def setup_default_users_and_groups():
    PortalMember.objects.create(username="regular_user")
    PortalMember.objects.create(username="test_applicant")
    PortalMember.objects.create(username="access_call_board_member")
    PortalMember.objects.create(username="access_call_coordinator")
    PortalMember.objects.create(username="consortium_admin")

    Group.objects.create(name="consortium_admins")
    consortium_admins = Group.objects.get(name="consortium_admins")
    consortium_admin = PortalMember.objects.get(username="consortium_admin")
    consortium_admin.groups.add(consortium_admins)
    consortium_admin.save()


def setup_access_call():
    coordinator = PortalMember.objects.get(username="access_call_coordinator")
    access_call_board_member = PortalMember.objects.get(
        username="access_call_board_member"
    )

    access_call = AccessCall.objects.create(
        title="Test access call",
        description="Description of access call",
        status="OPEN",
        closing_date="2024-11-11",
        coordinator=coordinator,
        board_chair=access_call_board_member,
    )

    access_call.board_members.set([access_call_board_member])

    add_group_permissions(
        "consortium_admins", AccessCall, ["change_accesscall", "add_accesscall"]
    )
    return access_call


def setup_application(call, applicant="test_applicant"):
    applicant = PortalMember.objects.get(username=applicant)
    AccessApplication.objects.create(applicant=applicant, call=call)


def add_group_permissions(group_name: str, model: type, permissions: list):

    group = Group.objects.get(name=group_name)
    content_type = ContentType.objects.get_for_model(model)
    for permission_name in permissions:
        permission = Permission.objects.get(
            codename=permission_name, content_type=content_type
        )
        group.permissions.add(permission)
    group.save()


class AuthAPITestCase(APITestCase):
    def authenticate(self, username: str):
        user = User.objects.get(username=username)
        self.client.force_authenticate(user=user)

    def deauthenticate(self):
        self.client.force_authenticate(user=None)

    def authenticated_op(self, username: str, op):
        self.authenticate(username)
        response = op()
        self.deauthenticate()
        return response

    def do_list(self):
        return self.client.get(self.url, format="json")

    def authenticated_list(self, username: str):
        return self.authenticated_op(username, self.do_list)

    def detail(self, resource_id: int):
        return self.client.get(self.url + f"{resource_id}/", format="json")

    def authenticated_detail(self, username: str, resource_id: int):
        return self.authenticated_op(username, partial(self.detail, resource_id))

    def create(self, data: dict, format="json"):
        return self.client.post(self.url, data, format=format)

    def update(self, data: dict, format="json"):
        return self.client.put(self.url, data, format=format)

    def put_file(self, resource_id: int, field: str, data, filename):
        return self.client.put(
            f"{self.url}{resource_id}/{field}/upload",
            data,
            headers={"Content-Disposition": f'attachment; filename="{filename}"'},
        )

    def authenticated_create(self, username: str, data: dict, format="json"):
        return self.authenticated_op(username, partial(self.create, data, format))

    def authenticated_put_file(
        self, username: str, resource_id: int, field: str, data, filename: str
    ):
        return self.authenticated_op(
            username, partial(self.put_file, resource_id, field, data, filename)
        )

    def assert_200(self, response):
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def assert_201(self, response):
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def assert_204(self, response):
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    def assert_401(self, response):
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def assert_403(self, response):
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)



from pathlib import Path
import os

SECRET_KEY = "fake-key"

MEDIA_ROOT = Path(__file__).parent / "test_output/mediafiles/"
os.makedirs(MEDIA_ROOT, exist_ok=True)

INSTALLED_APPS = [
    "tests",
    "api",
    "django.contrib.contenttypes",
    "django.contrib.auth",
    "django.contrib.sessions",
    "rest_framework.authtoken",
]

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": "db.sqlite3",
        "USER": "user",
        "PASSWORD": "password",
    }
}

ROOT_URLCONF = "api.urls"

REST_FRAMEWORK = {
    "DEFAULT_PERMISSION_CLASSES": [
        "rest_framework.permissions.IsAuthenticated",
    ],
    "DEFAULT_AUTHENTICATION_CLASSES": [
        "rest_framework.authentication.TokenAuthentication",
        "rest_framework.authentication.SessionAuthentication",
    ],
}



"""
Django settings
"""

from pathlib import Path

from ichec_django_core import settings
from ichec_django_core.settings import *  # NOQA

BASE_DIR = Path(__file__).resolve().parent.parent

ROOT_URLCONF = "marinerg_backend.urls"
WSGI_APPLICATION = "marinerg_backend.wsgi.application"
ASGI_APPLICATION = "marinerg_backend.asgi.application"

TEMPLATES = settings.get_templates(BASE_DIR)
DATABASES = settings.get_databases(BASE_DIR)

STATIC_ROOT = settings.get_static_root(BASE_DIR)
MEDIA_ROOT = settings.get_media_root(BASE_DIR)

INSTALLED_APPS = [
    "marinerg_test_access.apps.MarinergTestAccessConfig",
    "marinerg_data_access.apps.MarinergDataAccessConfig",
    "marinerg_facility.apps.MarinergFacilityConfig",
]
INSTALLED_APPS.extend(settings.INSTALLED_APPS)
